import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { DeleteResult, Repository } from 'typeorm';
import { Album } from '../album/album.entity';
import { Song } from './song.entity';
import { CreateSongDto } from './dto/create-song.dto';
import { UpdateSongDto } from './dto/update-song.dto';

@Injectable()
export class SongService {
  constructor(
    @InjectRepository(Song)
    private songRepository: Repository<Song>,
    @InjectRepository(Album)
    private albumRepository: Repository<Album>
  ) {}

  findAll(): Promise<Song[]> {
    return this.songRepository.find({
      order: {
        trackNumber: "ASC"
      }
    });
  }

  findOneById(id: number): Promise<Song | null> {
    return this.songRepository.findOneBy({ id: id });
  }

  async create(createSongDto: CreateSongDto): Promise<Song | null> {
    const album = await this.albumRepository.findOneBy({id: createSongDto.albumId});
    if (album) {
      const song = new Song();
      song.title = createSongDto.title;
      song.duration = createSongDto.duration;
      song.trackNumber = createSongDto.trackNumber;
      song.album = album;
      const savedSong = await this.songRepository.save(song);
      return savedSong;
    } else {
      throw new Error(`Unable to find Album with ID ${createSongDto.albumId}`);
    }
  }

  async update(id: number, updateSongDto: UpdateSongDto): Promise<Song | null> {
    if (id === updateSongDto.id) {
      const album = await this.albumRepository.findOneBy({id: updateSongDto.albumId});
      const songToUpdate = await this.songRepository.findOneBy({ id: updateSongDto.id });
      if (!songToUpdate || !album)
        return null;

      songToUpdate.id = updateSongDto.id;
      songToUpdate.title = updateSongDto.title;
      songToUpdate.duration = updateSongDto.duration;
      songToUpdate.trackNumber = updateSongDto.trackNumber;
      songToUpdate.album = album;
      const song = await this.songRepository.save(songToUpdate);
      return song;
    } else {
      return null;
    }
  }

  async remove(id: number): Promise<DeleteResult> {
    return await this.songRepository.delete(id);
  }

}
